require 'execjs'

# Utilities which we use to handle differences between various execjs runtimes
module ExecJS::Rails::RuntimeSupport
  # This should never actually be thrown, it is just a way to keep our rescue
  # blocks valid when there is no runtime configured. This way, rather than
  # getting a mysterious 'class or module required for rescue clause' error,
  # we get the good 'Could not find a JavaScript runtime.' error.
  class DisabledError < ::StandardError; end

  def self.using_runtime(klass)
    ExecJS::Runtimes.autodetect.class == klass
  end

  def self.using_v8
    using_runtime(ExecJS::RubyRacerRuntime)
  end

  def self.using_rhino
    using_runtime(ExecJS::RubyRhinoRuntime)
  end

  def self.error_class
    if using_v8
      V8::Error
    elsif using_rhino
      Rhino::JSError
    else
      DisabledError
    end
  end
end
